/** 
 *  @file QuickSort.java Quicksort Algorithmus
 *  nach Aho, Hopcroft und Ulmann
 */


import java.util.Random;

class QuickSort {



    /**
     *  Prueffunktion fuer die korrekte aufsteigende 
     *  Sortierung eines Arrays.
     *
     *  @param a Zu pruefender Array.
     *
     *  @return true, falls der Array korrekt aufsteigend sortiert ist. <br>
     *          false sonst.
     */
    static boolean isSorted(int[] a) {

	for ( int i = 1; i < a.length; i++ ) {
	    if ( a[i-1] > a[i] ) return false;
	}

	return true;

    }

    /**
     *  Vertauschung der Werte
     *  eines Arrays an den Indexwerten i und j.
     *
     *  @param a Array, in dem die Werte a[i] und a[j] zu vertauschen sind.
     *
     *  @param i Erster index; dieser muss im Wertebereich 0..a.length-1 liegen.
     *
     *  @param j Zweiter index; dieser muss im Wertebereich 0..a.length-1 liegen.
     */
    static void swap(int[] a, int i, int j) {
	int aux = a[i];
	a[i] = a[j];
	a[j] = aux;
    }

    /**
     *  Ermittlung eines Pivotelementes
     *            in den Grenzen ug..og des Arrays a,
     *            welches NICHT das kleinste Element
     *            in diesem Bereich ist.
     *  
     *  @param a Array, in dessen Grenzen ug, og 
     *           das Pivo-Element zu suchen ist
     *
     *  @param ug Untergrenze des Suchbereichs
     *            Wertebereich 0..a.length-1
     *
     *  @param og Obergrenze des Suchbereichs
     *            Wertebereich 0..a.length-1
     *            Es muss ug <= og gelten.
     *
     *  @return -1 Wenn alle Elemente im Bereich
     *             ug..og den selben Wert haben <br>
     *          i  Wenn a[i] NICHT den kleinsten Wert
     *             von a im Bereich  ug..og hat
     */
    static int getPivotIdx(int[] a, int ug, int og) {
	int aux = a[ug];
	for ( int i = ug; i <= og; i++ ) {
      
            // Wenn a[i] nicht das kleinste Element
            // ist, wird dessen Index zurueck gegeben.
	    if ( aux < a[i] ) 
		return i;
	    else if ( a[i] < aux )
		return ug;
	}
	return -1;
    }


    /** Hauptfunktion des Quicksort-Algorithmus.
     *
     *  @param a Array, in dessen Grenzen ug..og der
     *           Algorithmus anzuwenden ist.
     *
     *  @param ug Untergrenze des Suchbereichs.
     *            Wertebereich 0..a.length-1
     *
     *  @param og Obergrenze des Suchbereichs.
     *            Wertebereich 0..a.length-1.
     *            Es muss ug <= og gelten.
     *
     */
    static void quicksort(int[] a, int ug, int og) {

        // Suche den Index eines geeigneten Pivot-Elementes.
	int pIdx = getPivotIdx(a,ug,og);

        // Wenn kein Pivot-Element gefunden wurde, bedeutet dies,
        // dass a[ug..og] lauter identische Werte enthaelt. Es ist also
        // im Bereich ug..og nichts mehr zu sortieren.
	if ( pIdx == -1 ) return;
 
        // p traegt den Wert des Pivot-Elementes
	int p = a[pIdx];

        // lo ist der Index fuer die Suche von links,
        // beginnend bei der Untergrenze ug.
	int lo = ug;

        // hi ist der Index fuer die Suche von rechts.
	int hi = og;
 
        // Ziel dieser while-Schleife: nach der Terminierung
        // der Schleife sind alle Elemente a[ug..lo-1] kleiner
        // als das Pivotelement p, und alle Elemengte a[lo..og]
        // sind groesser oder gleich p.
	while ( lo <= hi ) {

	    swap(a,lo,hi);
	    while ( a[lo] < p ) lo++;
	    while ( a[hi] >= p ) hi--;

	}

        // Teile-und-herrsche Prinzip: Wende den
        // Algorithmus rekursiv auf a[ug..lo-1]
        // und a[lo..og] an:
	quicksort(a,ug,lo-1);
 	quicksort(a,lo,og);

    }

    /**
     *  Fuelle array a mit Zufallszahlen.
     */
    static void fillArray(int[] a) {

	Random r = new Random(17);

	for ( int i = 0; i < a.length; i++ ) {
	    a[i] = r.nextInt();
	}

    }

    
    /**
     *  Drucke den Inhalt von a aus; 
     *  mit vorangestellter Ueberschrift s.
     */
    static void printArray(int[] a, String s) {

	System.out.println(s);

	for ( int i = 0; i < a.length; i++ ) {
	    System.out.println("a[" + i + "] = " + a[i]);
	}

    }

    /**
     *  Hauptprogramm fuer die Anwendung und
     *  den Test des Quicksort-Algorithmus.
     *
     *  @param args    args[0] enthaelt die Groesse des zum Sortieren
     *                 zu verwendenden Arrays (als String)
     */
    public static void main(String[] args) {

	Integer len = new Integer(args[0]);
	int length = len.intValue();
	int a[] = new int[length];

	fillArray(a);
	// printArray(a,"UNSORTIERT:");


	quicksort(a,0,length-1);

	// printArray(a,"SORTIERT:");


	if ( isSorted(a) ) {
	    System.out.println("PASS.");
	}
	else {
	    System.out.println("FAIL.");
	}


    }


}
